// app/extend/helper.js
const crypto = require('crypto')

module.exports = {
    /**
     * 创建一个带有错误码的Error对象
     * @param msg 错误信息
     * @param code 错误码，默认为 errCode.APP_ERROR_CODE
     * @param arg_props 其他混入到Error对象的属性
     * @returns {Error}
     */
    createError(msg, code, arg_props) {
        let props = arg_props
        const err = msg instanceof Error ? msg : new Error(msg)

        switch (true) {
            // 没有传code的情况
            case typeof code === 'object':
                props = code
                break
            case !!code:
                err.code = this.config.errCode.APP_ERROR_CODE + code
                break
            default:
                // 全零为未登记的错误码
                err.code = this.config.errCode.APP_ERROR_CODE + this.config.errCode.NOT_REGISTER_ERROR
        }

        if (props) {
            for (let key in props) {
                if (props.hasOwnProperty(key)) {
                    err[key] = props[key]
                }
            }
        }
        return err
    },

    /**
     * 带有 key 的md5加密，单向加密
     * @param data 要加密的数据
     * @param key  秘钥
     * @returns {string}
     */
    md5withKey(data, key) {
        const hash = crypto.createHash('md5')
        hash.update(data)
        hash.update(key)
        return hash.digest('hex')
    },

    /**
     * 加密函数
     * @param type  加密类型，默认为 aes-256-cbc
     * @param data  要加密的数据
     * @param key   aes-256-cbc时为 32位
     * @param iv    aes-256-cbc时为 16位
     * @returns {string}
     */
    aesEncrypt({ type = 'aes-256-cbc', data, key, iv }) {
        const cipher = crypto.createCipheriv(type, key, iv)
        let crypted = cipher.update(data, 'utf8', 'hex')
        crypted += cipher.final('hex')
        return crypted
    },

    /**
     * 解密函数
     * @param type  解密类型，默认为 aes-256-cbc
     * @param data  要解密的数据
     * @param key   aes-256-cbc时为 32位
     * @param iv    aes-256-cbc时为 16位
     * @returns {string}
     */
    aesDecrypt({ type = 'aes-256-cbc', data, key, iv }) {
        const decipher = crypto.createDecipheriv(type, key, iv)
        let decrypted = decipher.update(data, 'hex', 'utf8')
        decrypted += decipher.final('utf8')
        return decrypted
    }
}